
' eschecs.bas

#include ".\libraries\mdlanguage\mdlanguage.bi"
#include ".\libraries\sgui\sgui.bas"
#include ".\libraries\sgui\simplegadget.bas"
#include ".\libraries\sgui\clickmapgadget.bas"
#include ".\libraries\sgui\togglegadget.bas"
#include ".\libraries\sgui\promotionbox.bas"
#include ".\libraries\iniobject\inifile.bi"

#include ".\chess\chess.bas"
#include ".\engine\engine.bas"
#include ".\files.bas"
#include ".\book.bas"

#define logPath ".\log\"
#define resPath ".\bitmap\"
#define buttonHeight 32
#define buttonWidth 156
#define boardLeft 8
#define boardTop 8

#ifndef false
#define false 0
#define true (not false)
#endif

const programName = "Eschecs"
const version = "0.8.9"
const optionsFile = "options.ini"
const redrawAll = true

declare sub Info(byval as string = "")
declare sub InfoSleep(byval as string)
declare function IsValidStartingSquare(byref as tChessGame, byval as string) as tBoolean
declare function IsPromotionMove(byref as tChessGame, byval as string) as tBoolean
declare sub LoadOptions(byref as string)
declare function ReplaceFieldName(byval as string) as string
declare sub HighlightMove(byval as integer, byval as integer, byval as string, byval as byte)

screenres 336, 456, 32,, 4

windowtitle programName
const textHeight = 14
dim as integer breit, hoch
screeninfo breit, hoch
width breit \ 8, hoch \ textHeight
#include "colors.bas"
#include "pieces_old.bas"
InitGFX
Info()

dim language as string
LoadOptions(language)

LoadCustomLanguage(default, language, "language\")
dim sNew as string = GetTranslation(default, "new")
dim sFlip as string = GetTranslation(default, "flip")
dim sQuit as string = GetTranslation(default, "quit")
dim sCheck as string = GetTranslation(default, "check")
dim sWhiteToMove as string = GetTranslation(default, "white")
dim sBlackToMove as string = GetTranslation(default, "black")
dim sComputerPlaysWhite as string = GetTranslation(default, "computer_white")
dim sComputerPlaysBlack as string = GetTranslation(default, "computer_black")
dim sState(gsCheckmate to gsDraw) as string
sState(gsCheckmate) = GetTranslation(default, StateToStr(gsCheckmate))
sState(gsStalemate) = GetTranslation(default, StateToStr(gsStalemate))
sState(gsDraw) = GetTranslation(default, StateToStr(gsDraw))
dim sWait as string = GetTranslation(default, "wait")

if language = "de" then
  sWhiteToMove = "Wei" & chr(225) & " am Zug"
  sComputerPlaysWhite = "Wei" & chr(225) & " automatisch"
end if

dim as eventHandle ptr event
dim as gadget ptr button1, button2, button3
dim as gadget ptr boardClickSensor
dim as gadget ptr toggle1, toggle2
dim as Gadget ptr list
#include "field.bas"

dim as byte computerPlaysWhite = false
dim as byte computerPlaysBlack = false

event = CreateEventHandle
button1 = AddSimpleGadget(event, 172, 336 + 0 * 40, buttonWidth, buttonHeight, sNew)
button2 = AddSimpleGadget(event, 008, 336 + 2 * 40, buttonWidth, buttonHeight, sFlip)
button3 = AddSimpleGadget(event, 172, 336 + 2 * 40, buttonWidth, buttonHeight, sQuit)
boardClickSensor = AddClickMapGadget(event, boardLeft, boardTop, resPath & "clickmap.bmp")
toggle1= AddToggleGadget(event, 008, 336 + 1 * 40, buttonWidth, buttonHeight, computerPlaysWhite, sComputerPlaysWhite)
toggle2= AddToggleGadget(event, 172, 336 + 1 * 40, buttonWidth, buttonHeight, computerPlaysBlack, sComputerPlaysBlack)

GadgetOn(button1, toggle2)

dim pieces as tChessPieces
pieces.Initialize()
color boardColor, boardBkColor
pieces.DrawBoard(8, 8, string(64, 32))

dim game as tChessGame
dim move as string = ""
dim state as tGameState = gsCheckmate
dim check as tBoolean = isFalse
dim bookMoveAvailable as integer

dim as tFileOfString history
dim as tFileOfString logFile

history.Initialize(logPath & "history.fen")
logFile.Initialize(logPath & "log.txt")

logFile.AddLine(String(80, "-"))
logFile.AddLine(programName & " " & version)
logFile.AddLine(String(80, "-"))
logFile.AddLine(date & " - " & time)
logFile.AddLine(String(80, "-"))

#ifdef CONSOLE
shell "echo " & programName & " " & version
shell "echo " & "Compiled with FreeBASIC " & __FB_VERSION__
#endif

dim upsideDown as byte = false

#macro UpdateAll()
game.GenerateGeometricalMoves()
game.GenerateSpecialMoves()
game.CheckMovesLegality()
state = game.GameState
if state = gsInProgress then
  with history
    if .Found(.LastLine()) = 3 then state = gsDraw
  end with
end if
pieces.DrawBoard(boardLeft, boardTop, game.String64, upsideDown, redrawAll)
if len(move) = 4 then HighlightMove(boardLeft, boardTop, move, upsideDown)
if state = gsInProgress then
  check = game.Check
  if check then InfoSleep(sCheck)
  Info(iif(game.activeColor = cWhite, sWhiteToMove, sBlackToMove))
else
  Info(sState(state))
end if
#endmacro

dim JSCP as tChessEngine
JSCP.minDepth = 4
rem The default value is 3.
JSCP.maxDepth = 6
rem The default value is 5.
dim engineFlag as byte

dim bookLoaded as byte = (LoadBook() = 0)
dim promotion as byte

do
  event->xSleep(1)
  if state = gsInProgress then
    if (computerPlaysWhite and (game.activeColor = cWhite)) _
    or (computerPlaysBlack and (game.activeColor = cBlack)) then
      bookMoveAvailable = false
      
      if BookLoaded then
        move = BookMove(game.EPD)
        
        if (len(move) = 4) andalso game.IsLegal(move) then
          bookMoveAvailable = true
          
          game.DoMove(move)
          
          JSCP.UserMove(move)
          
          logFile.AddLine(move & " (book)")
          history.AddLine(game.FEN)
          
#ifdef CONSOLE
          shell "echo " & move & " (book)"
#endif
          UpdateAll()
        else
          rem
        end if
        move = ""
      end if
      
      if bookMoveAvailable = false then
        Info(sWait)
        move = JSCP.EngineMove(engineFlag)
        
        if game.IsLegal(move) then
          game.DoMove(move)
          
          logFile.AddLine(move & " (engine)")
          logFile.AddLine("Flag = " & str(engineFlag))
          history.AddLine(game.FEN) 
#ifdef CONSOLE
          shell "echo " & move & " (engine)"
          shell "echo " & "Flag = " & str(engineFlag)
#endif

          UpdateAll()
        else
          logFile.AddLine("The engine returned a move estimated illegal (" & move & ").")
          logFile.AddLine("Engine flag = " & str(engineFlag))
          logFile.AddLine("Engine stopped.")
#ifdef CONSOLE
          shell "echo " & "The engine returned a move estimated illegal (" & move & ")."
          shell "echo " & "Engine flag = " & str(engineFlag)
          shell "echo " & "Engine stopped."
#endif
          computerPlaysWhite = false
          computerPlaysBlack = false
        end if
        
        move = ""
      end if
    end if
  end if
  if event->GADGETMESSAGE then
    select case event->GADGETMESSAGE
      case button1
        game.Initialize()
        history.Initialize()
        
        logFile.AddLine("New game")
        
        history.AddLine(game.FEN)
#ifdef CONSOLE
        shell "echo " & "New game"
#endif

        UpdateAll()
        JSCP.Initialize
      case button2
        upsideDown = not upsideDown
        pieces.DrawBoard(boardLeft, boardTop, game.String64, upsideDown, redrawAll)
      case button3
        event->EXITEVENT = 1
      case boardClickSensor
        field_ID = getClickMapColor(boardClickSensor)
        
        if state = gsInProgress then
          if upsideDown then
            move += ReplaceFieldName(Board(field_ID).fieldname)
          else
            move += Board(field_ID).fieldname
          end if
        end if
        
        if len(move) = 2 then
          if IsValidStartingSquare(game, move) then
            Info()
          else
            beep
            Info(iif(game.activeColor = cWhite, sWhiteToMove, sBlackToMove))
            move = ""
          end if
        end if
        
        if len(move) = 4 then
          if game.IsLegal(move) then
            if IsPromotionMove(game, move) then
              select case language
                case "de"
                  promotion = PromotionBox(97, 139, "Umwandlung", "S", "L", "T", "D")
                case "en"
                  promotion = PromotionBox(97, 139)
                case "fr"
                  promotion = PromotionBox(97, 139, "Promotion", "C", "F", "T", "D")
              end select
              game.DoMove(move, promotion)
              select case promotion
                case cKnight
                  JSCP.UserMove(move, cKnightValue)
                case cBishop
                  JSCP.UserMove(move, cBishopValue)
                case cRook
                  JSCP.UserMove(move, cRookValue)
                case cQueen
                  JSCP.UserMove(move)
              end select
            else
              game.DoMove(move)
              JSCP.UserMove(move)
            end if
            
            logFile.AddLine(move & " (user)")
            history.AddLine(game.FEN)
#ifdef CONSOLE
            shell "echo " & move & " (user)"
#endif
            UpdateAll()
          else
            beep
            Info(iif(game.activeColor = cWhite, sWhiteToMove, sBlackToMove))
          end if
          move = ""
        end if
      case toggle1
        computerPlaysWhite = GetSelect(toggle1)
      case toggle2
        computerPlaysBlack = GetSelect(toggle2)
    end select
  end if
  if event->KEY<>"" then
    select case event->KEY
      case chr(27)
        event->EXITEVENT = 1
    end select
  end if
loop until event->EXITEVENT

DestroyEventHandle(event)
if bookLoaded then CloseBook()

end

sub Info(byval s as string = "")
  const x = 8
  const y = 336
  FillA(x, y, buttonWidth, buttonHeight, infoColor, 1)
  draw string _
    ((buttonWidth - 8 * len(s)) \ 2 + x, y + (buttonHeight - textHeight) \ 2), _
    left(s, 18), _
    infoTextColor
end sub

sub InfoSleep(byval s as string)
  Info(s)
  sleep 1500
end sub

function IsValidStartingSquare(byref g as tChessGame, byval m as string) as tBoolean
  return sgn(g.board(m[0] - 96, m[1] - 48)) = g.activeColor
end function

function IsPromotionMove(byref g as tChessGame, byval m as string) as tBoolean
  return (abs(g.board(m[0] - 96, m[1] - 48)) = cPawn) andalso ((m[3] - 48 = 1) or (m[3] - 48 = 8))
end function

sub LoadOptions(byref aLanguage as string)
  dim myIni as inifobj.iniobj = inifobj.iniobj(optionsFile)
  dim update as integer = false
  select case myIni.ReadString("interface", "language", "")
  	case "de"
  		aLanguage = "de"
  	case "en"
  		aLanguage = "en"
  	case "fr"
  		aLanguage = "fr"
  	case else
      myIni.WriteString("interface", "language", "en")
  		aLanguage = "en"
  		update = true
  end select
  if update then myIni.UpdateFile()
end sub

function ReplaceFieldName(byval s as string) as string
  dim as ubyte asc1 = s[0] - 96, asc2 = s[1] - 48
  asc1 = 9 - asc1
  asc2 = 9 - asc2
  return chr(asc1 + 96, asc2 + 48)
end function

sub HighlightMove( _
  byval xo as integer, _
  byval yo as integer, _
  byval m as string, _
  byval aUpsideDown as byte _
)
  const thickLine = false
  const highlightStartingSquare = false
  dim as integer x1, y1, x2, y2
  
  x1 = m[0] - 96
  y1 = m[1] - 48
  x2 = m[2] - 96
  y2 = m[3] - 48
  
  if not aUpsideDown then
    x1 = 40 * (x1 - 1) + xo
    y1 = 40 * (8 - y1) + yo
    x2 = 40 * (x2 - 1) + xo
    y2 = 40 * (8 - y2) + yo
  else
    x1 = 40 * (8 - x1) + xo
    y1 = 40 * (y1 - 1) + yo
    x2 = 40 * (8 - x2) + xo
    y2 = 40 * (y2 - 1) + yo
  end if
  
  if highlightStartingSquare then line(x1, y1)-(x1 + 39, y1 + 39), highlightColor, b
  line(x2, y2)-(x2 + 39, y2 + 39), highlightColor, b
  
  if thickLine then
    if highlightStartingSquare then line(x1 + 1, y1 + 1)-(x1 + 38, y1 + 38), highlightColor, b
    line(x2 + 1, y2 + 1)-(x2 + 38, y2 + 38), highlightColor, b
  end if
end sub
